@@ -1,11 +1,9 @@
 """main port-for functionality."""
 
-import contextlib
-import errno
 import random
 import socket
 from itertools import chain
-from typing import Iterable, List, Optional, Set, Tuple, Type, TypeVar, Union
+from typing import Iterable, Type, TypeVar
 
 from port_for import ephemeral, utils
 
@@ -16,8 +14,8 @@ SYSTEM_PORT_RANGE = (0, 1024)
 
 
 def select_random(
-    ports: Optional[Set[int]] = None,
-    exclude_ports: Optional[Iterable[int]] = None,
+    ports: set[int] | None = None,
+    exclude_ports: Iterable[int] | None = None,
 ) -> int:
     """Return random unused port number."""
     if ports is None:
@@ -42,8 +40,8 @@ def is_available(port: int) -> bool:
 def available_ports(
     low: int = 1024,
     high: int = 65535,
-    exclude_ranges: Optional[List[Tuple[int, int]]] = None,
-) -> Set[int]:
+    exclude_ranges: list[tuple[int, int]] | None = None,
+) -> set[int]:
     """Return a set of possible ports.
 
     .. note::
@@ -73,8 +71,8 @@ def available_ports(
 
 
 def good_port_ranges(
-    ports: Optional[Set[int]] = None, min_range_len: int = 20, border: int = 3
-) -> List[Tuple[int, int]]:
+    ports: set[int] | None = None, min_range_len: int = 20, border: int = 3
+) -> list[tuple[int, int]]:
     """Return a list of 'good' port ranges.
 
     Such ranges are large and don't contain ephemeral or well-known ports.
@@ -85,76 +83,97 @@ def good_port_ranges(
         ports = available_ports()
     ranges = utils.to_ranges(list(ports))
     lenghts = sorted([(r[1] - r[0], r) for r in ranges], reverse=True)
-    long_ranges = [
-        length[1] for length in lenghts if length[0] >= min_range_len
-    ]
-    without_borders = [
-        (low + border, high - border) for low, high in long_ranges
-    ]
+    long_ranges = [length[1] for length in lenghts if length[0] >= min_range_len]
+    without_borders = [(low + border, high - border) for low, high in long_ranges]
     return without_borders
 
 
-def available_good_ports(min_range_len: int = 20, border: int = 3) -> Set[int]:
+def available_good_ports(min_range_len: int = 20, border: int = 3) -> set[int]:
     """List available good ports."""
-    return utils.ranges_to_set(
-        good_port_ranges(min_range_len=min_range_len, border=border)
-    )
+    return utils.ranges_to_set(good_port_ranges(min_range_len=min_range_len, border=border))
 
 
 def port_is_used(port: int, host: str = "127.0.0.1") -> bool:
     """Return if port is used.
 
-    Port is considered used if the current process
-    can't bind to it or the port doesn't refuse connections.
+    If we can connect to the port or we cannot bind to it, it's used.
     """
-    unused = _can_bind(port, host) and _refuses_connection(port, host)
-    return not unused
+    # Used if something is listening on the port, and we can connect to it
+    if _accepts_connection(port, host):
+        return True
+    # Used if we cannot bind to the port.
+    if not _can_bind(port, host):
+        return True
+    return False
 
 
 def _can_bind(port: int, host: str) -> bool:
-    sock = socket.socket()
-    with contextlib.closing(sock):
-        try:
-            sock.bind((host, port))
-        except socket.error:
-            return False
+    """Try binding on common addresses to detect if the port is free.
+
+    Some platforms (notably Windows) allow binding to 127.0.0.1 even when the
+    port is already bound on INADDR_ANY (0.0.0.0). To reliably detect usage, we
+    attempt to bind on both the requested host and INADDR_ANY. If either bind
+    fails, consider the port as used.
+    """
+    hosts_to_try = []
+    # Always try the requested host first
+    hosts_to_try.append(host)
+    # Also try INADDR_ANY for IPv4 to catch cases where 0.0.0.0 is occupied
+    if host not in ("", "0.0.0.0"):
+        hosts_to_try.append("")
+        hosts_to_try.append("0.0.0.0")
+
+    for h in hosts_to_try:
+        with socket.socket() as sock:
+            try:
+                sock.bind((h, port))
+            except socket.error:
+                return False
     return True
 
 
-def _refuses_connection(port: int, host: str) -> bool:
-    sock = socket.socket()
-    with contextlib.closing(sock):
+def _accepts_connection(port: int, host: str) -> bool:
+    """Return True if connect_ex succeeds (service is listening).
+
+    Works reliably across platforms, including Windows.
+    """
+    with socket.socket() as sock:
         sock.settimeout(1)
+        sock.setblocking(True)
         err = sock.connect_ex((host, port))
-        return err == errno.ECONNREFUSED
+        # Relying on ECONNREFUSED does not produce reliable results on windows,
+        # which could result in
+        # either ECONREFUSED (Mapped in windows to `WSAECONNREFUSED`),
+        # timeout or return any other error.
+        return err == 0
 
 
 T = TypeVar("T")
 
 
-def filter_by_type(lst: Iterable, type_of: Type[T]) -> List[T]:
+def filter_by_type(lst: Iterable, type_of: Type[T]) -> list[T]:
     """Return a list of elements with given type."""
     return [e for e in lst if isinstance(e, type_of)]
 
 
-PortType = Union[
-    str,
-    int,
-    Tuple[int, int],
-    Set[int],
-    List[str],
-    List[int],
-    List[Tuple[int, int]],
-    List[Set[int]],
-    List[Union[Set[int], Tuple[int, int]]],
-    List[Union[str, int, Tuple[int, int], Set[int]]],
-]
+PortType = (
+    str
+    | int
+    | tuple[int, int]
+    | set[int]
+    | list[str]
+    | list[int]
+    | list[tuple[int, int]]
+    | list[set[int]]
+    | list[set[int] | tuple[int, int]]
+    | list[str | int | tuple[int, int] | set[int]]
+)
 
 
 def get_port(
-    ports: Optional[PortType],
-    exclude_ports: Optional[Iterable[int]] = None,
-) -> Optional[int]:
+    ports: PortType | None,
+    exclude_ports: Iterable[int] | None = None,
+) -> int | None:
     """Retun a random available port.
 
     If there's only one port passed (e.g. 5000 or '5000') function
@@ -181,29 +200,29 @@ def get_port(
     except TypeError:
         pass
 
-    ports_set: Set[int] = set()
+    ports_set: set[int] = set()
 
     try:
         if not isinstance(ports, list):
             ports = [ports]
-        ranges: Set[int] = utils.ranges_to_set(filter_by_type(ports, tuple))
-        nums: Set[int] = set(filter_by_type(ports, int))
-        sets: Set[int] = set(
+        ranges: set[int] = utils.ranges_to_set(filter_by_type(ports, tuple))
+        nums: set[int] = set(filter_by_type(ports, int))
+        sets: set[int] = set(
             chain(
                 *filter_by_type(
-                    ports, (set, frozenset)  # type: ignore[arg-type]
+                    ports,
+                    (set, frozenset),  # type: ignore[arg-type]
                 )
             )
         )
         ports_set = ports_set.union(ranges, sets, nums)
     except ValueError:
         raise PortForException(
-            "Unknown format of ports: %s.\n"
+            f"Unknown format of ports: {ports}.\n"
             'You should provide a ports range "[(4000,5000)]"'
             'or "(4000,5000)" or a comma-separated ports set'
             '"[{4000,5000,6000}]" or list of ints "[400,5000,6000,8000]"'
             'or all of them "[(20000, 30000), {48889, 50121}, 4000, 4004]"'
-            % (ports,)
         )
 
     return select_random(ports_set, exclude_ports)
